#!/bin/sh

# This helper script will handle idempotently creating a localstack instance running all the services we use commonly
# Since we expect to run a single instance of localstack for all our mocking purposes, we use a well known
# name for the Deployment and Service of "localstack"

# TODO - Consider moving this over to a low-level cluster-wide infra chart that is installed once to
#        set up namespaces and other singletons.  It could be conditional on if the cluster is AWS or not.
#        As currently implemented, this doesn't get cleaned up on uninstall, so it leaves cruft lying around.

# TODO we might consider hijacking the regular AWS flows and creating services to access the APIs in the future
# in which case this "localstack" assumption for the Service name in the target namespace may no longer be appropriate
# and instead we'll want a more specific service name for each use-case to allow finer grain authz controls

# TODO this list is definitely not complete...
source scripts/utils.sh
set -e

# es == elasticsearch
SERVICES=dynamodb,kinesis,firehose,s3,es,iam,sts,sqs,sns,lambda,stepfunctions


if [ -z "${appsNamespace}" ] ; then
    echo "Must set env appsNamespace"
    exit 1
fi

if [ -z "${infraNamespace}" ] ; then
    echo "Must set env infraNamespace"
    exit 1
fi

# We leverage a yaml and `kubectl apply` to ensure this is idempotently created
# This makes it easier to have multiple charts leverage this and all share the same resource
# If we modeled it as a standard resource, helm would complain if we had 2 or more charts that
# think they own the same resource
cat > /tmp/localstack.yaml <<EOF
apiVersion: apps/v1
kind: Deployment
metadata:
  name: localstack
spec:
  selector:
    matchLabels:
      app: localstack
  replicas: 1
  strategy:
    type: Recreate
  template:
    metadata:
      labels:
        app: localstack
    spec:
      containers:
      - image: ${registryPrefixDockerhub}/dockerhub-proxy-cache/localstack/localstack:0.12.16
        imagePullPolicy: IfNotPresent
        name: localstack
        securityContext:
          capabilities:
            drop:
              - ALL
          runAsNonRoot: true
          runAsUser: 1001
          runAsGroup: 1001
          privileged: false
        ports:
          - name: api
            containerPort: 4566
          - name: ui
            containerPort: 8080
        env:
          - name: SERVICES
            value: "${SERVICES}"
          - name: DATA_DIR
            value: "/tmp/localstack/data"
          - name: KINESIS_PROVIDER
            value: kinesalite
          - name: DEBUG
            value: "1"
          - name: HOSTNAME_EXTERNAL
            value: "localstack"
        livenessProbe:
          failureThreshold: 5
          periodSeconds: 10
          successThreshold: 1
          timeoutSeconds: 1
          httpGet:
            path: /health
            port: api
        readinessProbe:
          exec:
            command:
              - "/bin/sh"
              - "-c"
              - "awslocal kinesis list-streams && awslocal s3api list-buckets"
          initialDelaySeconds: 10
          periodSeconds: 30
          timeoutSeconds: 10
          failureThreshold: 6
          successThreshold: 1
        volumeMounts:
          - name: localstack-data
            mountPath: /tmp/localstack
      restartPolicy: Always
      volumes:
        - name: localstack-data
          emptyDir: {}
---
apiVersion: v1
kind: Service
metadata:
  name: localstack
spec:
  type: ClusterIP
  selector:
    app: localstack
  ports:
    - name: api
      port: 4566
      targetPort: api
    - name: ui
      port: 8080
      targetPort: ui
EOF

# Now create the ExternalName service pointing back to the localstack service
cat > /tmp/externalservice.yaml <<EOF
kind: Service
apiVersion: v1
metadata:
  name: "localstack"
  namespace: ${appsNamespace}
spec:
  type: ExternalName
  externalName: "localstack.${infraNamespace}.svc.cluster.local"
EOF

# Now create the ExternalName configmap pointing back to the localstack service
cat > /tmp/externalhostnameconfig.yaml <<EOF
kind: ConfigMap
apiVersion: v1
metadata:
  name: localstack-endpoint
  namespace: ${appsNamespace}
data:
  endpoint: "localstack.${infraNamespace}.svc.cluster.local"
EOF

set -e

# Dump out the YAML we're going to create to help debugging if something goes bad
run_cmd nl /tmp/localstack.yaml

echo "Creating (or updating) localstack"
run_cmd kubectl apply -f /tmp/localstack.yaml --wait=true

# Dump out the YAML we're going to create to help debugging if something goes bad
run_cmd nl /tmp/externalservice.yaml

echo "Creating (or updating) service in ${appsNamespace}"
run_cmd kubectl apply -f /tmp/externalservice.yaml --wait=true

echo "Creating (or updating) configmap in ${appsNamespace}"
run_cmd kubectl apply -f /tmp/externalhostnameconfig.yaml --wait=true

# # Note: the waits don't seem to actually wait for it to be healthy, make sure it responds before moving on...
# while ! aws --endpoint-url=http://localstack:4566 s3 ls ; do
#   echo "$(date) Waiting for localstack to come up..."
#   sleep 2
# done
echo "Wail for localstack to come up ( timeout 300 sec , interval 5 secs )...."
wait-until -t 300 -i 5 true aws --endpoint-url=http://localstack:4566 s3 ls

echo "$(date) done"
